<?php

declare(strict_types=1);

namespace app\models;

use yii;

use app\models\traits\SearchByEdge;

use app\models\Account;

use Exception;

/**
 * Поиск
 *
 * @see Product Поиск по товарам
 */
class Notification extends Document
{
    use SearchByEdge;

    /**
     * Сценарий для доверенного пользователя с созданием уведомления
     */
    const SCENARIO_TRUSTED_CREATE = 'create';

    /**
     * Тип уведомления: памятка
     */
    const TYPE_NOTICE = 'notice';

    /**
     * Тип уведомления: предупреждение
     */
    const TYPE_WARNING = 'warning';

    /**
     * Цель для отправки уведомления
     *
     * Расшифровывается как $target
     *
     * @see SCENARIO_TRUSTED_CREATE
     */
    public Account|string|array|null $account;

    /**
     * Текст уведомления
     */
    public string $text;

    /**
     * Типы уведомлений
     */
    public array $typs = [
        'notice' => 'Уведомление',
        'warning' => 'Предупреждение'
    ];

    /**
     * Имя коллекции
     */
    public static function collectionName(): string
    {
        return 'notification';
    }

    /**
     * Свойства
     */
    public function attributes(): array
    {
        return array_merge(
            parent::attributes(),
            [
                'html',
                'type'
            ]
        );
    }

    /**
     * Метки свойств
     */
    public function attributeLabels(): array
    {
        return array_merge(
            parent::attributeLabels(),
            [
                'html' => 'HTML',
                'type' => 'Тип',
                'trgt' => 'Получатели',
                'text' => 'Текст',
            ]
        );
    }

    /**
     * Правила
     */
    public function rules(): array
    {
        return array_merge(
            parent::rules(),
            [
                [
                    'html',
                    'required'
                ],
                [
                    'type',
                    'default',
                    'value' => 'notice'
                ]
            ]
        );
    }

    /**
     * Запись
     *
     * @param bool|string $html Содержимое уведомления (HTML или текст)
     * @param Account|array|string|null $account Получатель уведомления
     */
    public function write(Account|array|string|null $account = null): self|array|null
    {
        return $this::_write($this->text, $this->html, $account, $this->type);
    }

    /**
     * Запись
     *
     * @param string $html Содержимое уведомления (HTML или текст)
     * @param bool|string|null $html Содержимое уведомления (HTML или текст)
     * @param Account|array|string|null $account Получатель уведомления
     * @param string $type Тип уведомления
     */
    public static function _write(string $text, bool|string|null $html = false, Account|array|string|null $account = null, string $type = self::TYPE_NOTICE): self|array|null
    {
        // Инициализация
        $model = new self;
        $account ?? $account = yii::$app->user->identity ?? throw new Exception('Не удалось инициализировать получателя');

        if ((bool) (int) $html) {
            // Получен текст в формете HTML-кода

            $model->html = $text ?? null;
        } else {
            // Получен необработанный текст

            $text = htmlspecialchars(strip_tags($text ?? null));

            $model->html = <<<HTML
                <p class="my-2 mx-3">$text</p>
            HTML;
        }

        if ($model->save()) {
            // Уведомление записано

            // Инициализация получателей и создание ребра
            if (empty($account)) {
                // Получатель не передан

                goto test;
            } else if (is_string($account)) {
                // Передана необработанная строка

                // Инициализация
                $delimiter = ',';

                // Конвертация
                $account = array_map('trim', explode($delimiter, $account));


                if (in_array('@all', $account, true)) {
                    // Найден флаг обозначающий отправку всем пользователям

                    // Инициализация
                    $return = [];

                    foreach (Account::readAll() as $account) {
                        // Перебор всех аккаунтов

                        // Запись ребра: УВЕДОМЛЕНИЕ -> АККАУНТ
                        $return[] = AccountEdgeNotification::writeSafe($model->readId(), $account->readId(), $type) ? $model : null;
                    }

                    return $return ? $return : null;
                }

                if (in_array('@test', $account, true)) {
                    // Найден флаг обозначающий тестирование (отправка самому себе)

                    test:
                    return AccountEdgeNotification::writeSafe($model->readId(), yii::$app->user->id, $type) ? $model : null;
                }
            }

            if (is_array($account)) {
                // Несколько получателей

                // Инициализация
                $return = [];

                foreach ($account as $account) {
                    // Перебор получателей

                    if ($account instanceof Account) {
                        // Один получатель

                        // Запись ребра: УВЕДОМЛЕНИЕ -> АККАУНТ
                        return AccountEdgeNotification::writeSafe($model->readId(), $account->readId(), $type) ? $model : null;
                    }

                    if ($account = Account::searchById(Account::collectionName() . '/' . $account)) {
                        // Аккаунт найден

                        echo ($account->readId()) . "\n";

                        // Запись ребра: УВЕДОМЛЕНИЕ -> АККАУНТ
                        $return[] = AccountEdgeNotification::writeSafe($model->readId(), $account->readId(), $type) ? $model : null;
                    }
                }

                return $return ? $return : null;
            } else if ($account instanceof Account) {
                // Один получатель

                // Запись ребра: УВЕДОМЛЕНИЕ -> АККАУНТ
                return AccountEdgeNotification::writeSafe($model->readId(), $account->readId(), $type) ? $model : null;
            }
        }

        return null;
    }
}
